package fr.soleil.docking.vl.view;

import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import javax.swing.Icon;

import com.vldocking.swing.docking.DockKey;
import com.vldocking.swing.docking.Dockable;
import com.vldocking.swing.docking.event.DockingActionCloseEvent;
import com.vldocking.swing.docking.event.DockingActionEvent;
import com.vldocking.swing.docking.event.DockingActionListener;

import fr.soleil.docking.view.IView;
import fr.soleil.docking.view.IViewListener;
import fr.soleil.docking.vl.SoleilDockingDesktop;

public class VlDockView implements IView, Dockable, DockingActionListener, FocusListener {
    protected Object id;
    protected boolean enabled;
    protected boolean visible;
    protected String title;
    protected Component component;
    protected Icon icon;
    protected DockKey key;
    protected SoleilDockingDesktop dockingDesktop;
    protected List<IViewListener> viewListeners;


    public VlDockView(String title, Icon icon, Component component, Object id) {
        this.title = title;
        this.icon = icon;
        this.component = component;
        this.id = id;
        if (id instanceof String){
            String stringId = (String) id;
            this.key = new DockKey(stringId, title, title, icon);
        } else {
           this.key = new DockKey(title, title, title, icon); 
        }
        
        this.viewListeners = new ArrayList<IViewListener>();
        this.component.addFocusListener(this);
    }

    public void setDockingDesktop(SoleilDockingDesktop dockingDesktop) {
        this.dockingDesktop = dockingDesktop;
        dockingDesktop.registerDockable(this);
    }

    @Override
    public Component getComponent() {
        return component;
    }

    @Override
    public Object getId() {
        return id;
    }

    @Override
    public boolean isEnabled() {
        return enabled;
    }

    @Override
    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    @Override
    public boolean isVisible() {
        return visible;
    }

    @Override
    public void setVisible(boolean visible) {
        this.visible = visible;
        if (visible) {
            dockingDesktop.showDockable(this);
        }
        else {
            dockingDesktop.close(this);
        }
        component.setVisible(visible);
        component.setEnabled(visible);
    }

    @Override
    public String getTitle() {
        return title;
    }

    @Override
    public Icon getIcon() {
        return icon;
    }

    @Override
    public Color getViewBackground() {
        return component.getBackground();
    }

    @Override
    public void setViewBackground(Color bg) {
        if (component instanceof Container) {
            List<Component> components = getAllComponents((Container) component);
            setComponentsBackground(components, bg);
        }
        component.setBackground(bg);
    }

    @Override
    public Color getViewForeground() {
        return component.getForeground();
    }

    @Override
    public void setViewForeground(Color fg) {
        component.setForeground(fg);
    }

    /*
     * Sets the background of all {@link Component}s in a given {@link List}
     * 
     * @param components The {@link List}
     * 
     * @param bg The background to set
     */
    protected void setComponentsBackground(List<?> components, Color bg) {
        if (components != null) {
            for (Object comp : components) {
                if (comp instanceof Component) {
                    ((Component) comp).setBackground(bg);
                }
            }
        }
    }

    /**
     * Sets the foreground of all {@link Component}s in a given {@link List}
     * 
     * @param components The {@link List}
     * @param bg The foreground to set
     */
    protected void setComponentsForeground(List<?> components, Color bg) {
        if (components != null) {
            for (Object comp : components) {
                if (comp instanceof Component) {
                    ((Component) comp).setForeground(bg);
                }
            }
        }
    }

    public List<Component> getAllComponents(final Container c) {
        Component[] comps = c.getComponents();
        List<Component> compList = new ArrayList<Component>();
        for (Component comp : comps) {
            compList.add(comp);
            if (comp instanceof Container) {
                compList.addAll(getAllComponents((Container) comp));
            }
        }
        return compList;
    }

    @Override
    public void setClosable(boolean closable) {
        key.setCloseEnabled(closable);

    }

    @Override
    public void addViewListener(final IViewListener listener) {
        viewListeners.add(listener);
    }

    @Override
    public DockKey getDockKey() {
        return key;
    }

    @Override
    public boolean acceptDockingAction(DockingActionEvent event) {
        return true;
    }

    @Override
    public void dockingActionPerformed(DockingActionEvent event) {
        if (event.getActionType() == DockingActionEvent.ACTION_CLOSE) {
            Dockable closedDockable = ((DockingActionCloseEvent) event).getDockable();
            if (getDockKey().getKey().equals(closedDockable.getDockKey().getKey())) {
                for (Iterator<IViewListener> listenersIterator = viewListeners.iterator(); listenersIterator
                        .hasNext();) {
                    IViewListener listener = listenersIterator.next();
                    listener.viewClosed();
                }
            }
        }
    }

    @Override
    public void focusGained(FocusEvent e) {
        for (IViewListener listener : viewListeners) {
            listener.focusGained();
        }
    }

    @Override
    public void focusLost(FocusEvent e) {
        // Not managed.
    }
}
 